import { APIClient } from "../ApiClient";
import { Family } from "../FamilyApi";
import moment from "moment";

export interface AccommodationReservation {
  id: number;
  family_id: number;
  accommodation_id: number;
  user_id: number;
  time_create: number;
  time_update: number;
  reservation_start: number;
  reservation_end: number;
  validated?: boolean;
}

export class AccommodationsReservationsList {
  private list: AccommodationReservation[];
  private map: Map<number, AccommodationReservation>;

  constructor(list: AccommodationReservation[]) {
    this.list = list;
    this.map = new Map();

    for (const m of list) {
      this.map.set(m.id, m);
    }

    this.list.sort((a, b) => a.reservation_start - b.reservation_start);
  }

  public get isEmpty(): boolean {
    return this.list.length === 0;
  }

  public get size(): number {
    return this.list.length;
  }

  public get fullList(): AccommodationReservation[] {
    return this.list;
  }

  filter(
    predicate: (m: AccommodationReservation) => boolean
  ): AccommodationReservation[] {
    return this.list.filter(predicate);
  }

  forAccommodation(id: number): AccommodationReservation[] {
    return this.filter((a) => a.accommodation_id === id);
  }

  get(id: number): AccommodationReservation | undefined {
    return this.map.get(id);
  }
}

export class AccommodationsReservationsApi {
  /**
   * Get the entire list of accommodations of a family
   */
  static async FullListOfFamily(
    family: Family
  ): Promise<AccommodationsReservationsList> {
    const data = (
      await APIClient.exec({
        method: "GET",
        uri: `/family/${family.family_id}/accommodations/reservations/full_list`,
      })
    ).data;

    return new AccommodationsReservationsList(data);
  }
}
